/*
 * Copyright (C) 2004 emuWorks
 * http://games.technoplaza.net/
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package chtdecoder.datastructures;

import chtdecoder.exceptions.InvalidAddressException;
import chtdecoder.exceptions.InvalidCHTCodeDataException;
import chtdecoder.exceptions.InvalidValueException;
import chtdecoder.exceptions.NameTooLongException;

/**
 * An abstract implementation of a CHTCode.
 *
 * @author John David Ratliff
 * @version 1.1, 09/14/04
 */
public abstract class AbstractCHTCode implements CHTCode {
    private boolean active = false;
    private int address;
    private int value;
    private String name;

    /**
     * A factory method for creating CHTCode objects.
     *
     * @param name The name of the CHTCode.
     * @param address The address of the CHTCode.
     * @param value The value of the CHTCode.
     * @param active Whether this code is active or not.
     *
     * @return Returns a CHTCode object.
     */
    public static CHTCode createCHTCode(String name, int address, int value,
                                        boolean active) {
        return new BasicCHTCode(name, address, value, active);
    }

    /**
     * Parses a 28-byte raw CHT bitstring into a CHTCode.
     *
     * @param rawdata The raw CHT bitstring.
     *
     * @return The parsed CHTCode.
     *
     * @throws InvalidCHTCodeDataException if the rawdata isn't a valid raw
     *         CHT bitstring.
     */
    public static CHTCode parseCHTCode(byte[] rawdata)
                                throws InvalidCHTCodeDataException {
        boolean active = true;
        String name = "";
        int address = 0;
        int value = 0;

        try {
            // the active bit (4 = off, 0 = on)
            if (rawdata[0] == 4) {
                active = false;
            }

            // the address (backwards on low-endian x86 systems)
            // hope this doesn't change
            // possible future bug if it does 
            // (or if it gets ported to a high-endian system)
            for (int pos = 4; pos > 1; pos--) {
                address <<= 8;
                address |= (((int)rawdata[pos]) & 0xFF);
            }

            // the value 
            value = ((int)rawdata[1]) & 0xFF;

            // the name (18 characters max)
            for (int pos = 8; pos < 26; pos++) {
                if (rawdata[pos] == 0) {
                    break;
                }

                name += (char)rawdata[pos];
            }

            return new BasicCHTCode(name, address, value, active);
        } catch (ArrayIndexOutOfBoundsException e) {
            throw new InvalidCHTCodeDataException();
        }
    }

    /**
     * Asks if this CHTCode is active (turned ON).
     *
     * @return true if this CHTCode is active; false otherwise.
     */
    public boolean isActive() {
        return this.active;
    }

    /**
     * Sets whether this CHTCode is active or not.
     *
     * @param active true for active (ON); false for inactive (OFF).
     */
    public void setActive(boolean active) {
        this.active = active;
    }

    /**
     * Gets the address of this CHTCode.
     *
     * @return The address.
     */
    public int getAddress() {
        return this.address;
    }

    /**
     * Sets the address of this CHTCode.
     *
     * @param address The address.
     *
     * @throws InvalidAddressException if the address is outside the range
     *         0-16777215 (2^24 - 1).
     */
    public void setAddress(int address) throws InvalidAddressException {
        if ((0xFF000000 & address) != 0) {
            throw new InvalidAddressException();
        }

        this.address = address;
    }

    /**
     * Gets the value of this CHTCode.
     *
     * @return The value.
     */
    public int getValue() {
        return this.value;
    }

    /**
     * Sets the value of this CHTCode.
     *
     * @param value The value.
     *
     * @throws InvalidValueException if the value is outside the range 0-255
     *         (2^8 - 1).
     */
    public void setValue(int value) throws InvalidValueException {
        if ((value & 0xFFFFFF00) != 0) {
            throw new InvalidValueException();
        }

        this.value = value;
    }

    /**
     * Gets the name of this CHTCode.
     *
     * @return The name.
     */
    public String getName() {
        return this.name;
    }

    /**
     * Sets the name of this CHTCode (18 characters max).
     *
     * @param name The name.
     *
     * @throws NameTooLongException if the new name is longer than 18
     *         characters.
     */
    public void setName(String name) throws NameTooLongException {
        if (name.length() > 18) {
            throw new NameTooLongException();
        }

        this.name = name;
    }
}
